-- PROGRAM-NAME: ldqdbm.rex          BACKUP-REQUIRED: Yes            
-- AUTHOR: Tony Dycks                REVISED BY: Tony Dycks          
-- DATE-WRITTEN: May 2, 2020         DATE-REVISED: August 17, 2020     
-- LANGUAGE: IBM Net Rexx            VERSION NO.: 2.05              
-- PLATFORM: Windows/Linux (Console) VERSION NO.: Any With Java SDK 
--                                                                   
-- DESCRIPTION:                                                      
--    Load QDBM Records For The Command Line Arguments Specifing The Filepath of The      
--    QDBM Database File And The CSV File With Data.                                          
--
-- PRE-REQUISITE(S):
--   1. Installation of QDBM Library API Dev Library & Dependencies
--      sudo apt-get install libqdbm-dev
--      sudo yum install libqdbm-devel 
--   or Build of QDBM API Library Source
--      su (for /usr/local configure and make)
--      ./configure
--      make
--      make check
--      make install
--      cp /usr/local/lib/libqdbm* /usr/lib
--   2. Setup of QDBM Library For Java Package
--      sudo apt-get install qdbm-java  {Debian Linux}
--   or Build of QDBM Library Source For Java 
--      cd java
--      su
--      Modify configure.in File for $JAVA_HOME
--      ./configure
--      make
--      make check
--      make install
--      cp /usr/local/lib/*qdbm* /usr/lib
--      cp /usr/local/lib/qdbm.jar $JAVA_HOME/jre/lib/ext
--
-- REVISION HISTORY:
--   Version  Change Description
--   =======  ==============================================================
--     1.0    Original Implementation
--              T. Dycks -- May 2, 2020
--     1.1    Reworked QDBM Depot Hash Object Initialization and Open
--              T. Dycks -- August 11, 2020
--     1.2    Added Logic To Detect Whether Platform Is Linux or Windows
--              T. Dycks -- August 17, 2020
-- USAGE:                                                            
--   rexxj2.sh ldqdbm.rex <QDBM-Filename> <CSV-Filename> {Linux}
--   rexxj2.cmd istqdbm.rex <QDBM-Filename> <CSV-Filename> {Windows}
--                                                                   
-- USAGE EXAMPLE(S):                                                    
--   rexxj.sh ldqdbm.rex $HOME/qdbm/mensworldcupchamps.qdb 
--            $HOME/txtfiles/MensWorldCupWinners.txt <Enter> {Linux} 
--                                                                   
-- REFERENCES:                                                               
--   Specifications of QDBM for Java
--   Copdbmkeyight (C) 2000-2006 Mikio Hirabayashi
--   Last Update: Thu, 26 Oct 2006
--   Examples 
-- URL:
--   https://fallabs.com/qdbm/jspex.html#examples
-- ===========================================================================
--   Tutorials Point -- Rexx -- Parsing
-- URL:
--   https://www.tutorialspoint.com/rexx/rexx_parsing.htm
-- ===========================================================================
--   Tutorials Point -- Java -- Check If a File Exists in Java 
-- URL:
--   https://www.tutorialspoint.com/Check-if-a-file-exists-in-Java
-- ===========================================================================
--   CodeJava -- Coding Your Passion 
--   How to Execute Operating System Commands in Java
--   Written by  Nam Ha Minh -- Last Updated on 27 July 2019
-- URL:
--   https://www.codejava.net/java-se/file-io/execute-operating-system-commands-using-runtime-exec-methods
-- ======================================================================================
--   Windows Command Line -- Windows Commands, Batch files, Command prompt and PowerShell
--   Create Empty File
-- URL:
--   https://www.windows-commandline.com/create-empty-file/
--

if rxFuncQuery("BSF") then  /* no bsf4rexx support available yet, hence load Java */
do
   call rxFuncAdd "BsfLoadFuncs", "BSF4Rexx", "BsfLoadFuncs"
   call BsfLoadFuncs    /* load all BSF-functions                    */
   call BsfLoadJava     /* load the JVM and set up BSF4Rexx-support  */
end

if      BsfInvokedBy()=1 then say "This Rexx program was invoked by Java!"
else if BsfInvokedBy()=2 then say "This Rexx program was invoked by Rexx, JVM loaded by Rexx!"
else                          say "No JVM present, we got troubles ..."

parse arg argqdbmflnm argcsvflnm

-- Mainline Code --
call banner
call main
call endmsg
exit

-- Print Program Description banner
banner:
  say ' '
  say 'ldqdbm.rex -- BSF4Rexx List QDBM Key Values For QDBM File Entered On Command Line'
  say 'Version 1.2'
  say 'Written By: Tony Dycks'
  say 'Revised By: Tony Dycks'
  say 'Date Written: May 2, 2020'
  say 'Date Revised: August 17, 2020'
  say ' '
  say 'QDBM Filename: ' || argqdbmflnm
  say 'CSV Filename: ' || argcsvflnm
  say ' '
  return

createqdbmfile: procedure expose argqdbmflnm
  say 'Creating QDBM File ...'
  if getosname() = "Linux" then
    do
      touchcmd = 'touch ' || argqdbmflnm
      address 'bash' touchcmd
    end
  else
    do
      touchcmd = 'fsutil file createnew ' || argqdbmflnm ' 0'
      address 'cmd' touchcmd
    end
    
  say 'QDBM File Created ...'
  return
  
delqdbmfile: procedure expose argqdbmflnm
  say 'Deleting QDBM File ...'
  if getosname() = "Linux" then
    do
      delcmd = 'rm ' || argqdbmflnm
      address 'bash' delcmd
    end
  else
    do
      delcmd = 'del ' || argqdbmflnm
      address 'cmd' delcmd
    end
    
  say 'Deletion Completed ...'
  return
  
endmsg:
  say ' '
  say '>>> End Of Program --  ldqdbm.rex <<<'
  say ' '
  return

fileexists: procedure expose argqdbmflnm
  File = bsf.import('java.io.File')
  qdbmfile = File~new(argqdbmflnm)
    
  if qdbmfile~exists() then
    do
      say 'File Exists ...'
      call delqdbmfile(argqdbmflnm)
    end
    
  return

getosname: procedure
  system = bsf.loadClass("java.lang.System")
  return system~getProperty("os.name")

-- Mainline Routine --
main:
  reccnt = 0
  say 'Opening Quick DBM File ...'
  call fileexists(argqdbmflnm)
  -- Create A New File Using The touch OS System Command 
  say 'Initializing New QDBM File ...'
  call createqdbmfile(argqdbmflnm)
  incnt = 0
  reccnt = 0
  -- Instantiate a QDBM Depot Hash File Object
  Depot = bsf.import("qdbm.Depot")  /* Import Java Class "qdbm.Depot" */
  dbm = Depot~new(argqdbmflnm, Depot~OWRITER, -1)
  say
  
  reccnt = 0

  do while lines(argcsvflnm) <> 0   /* Loop as long as there are lines     */
    inln = linein(argcsvflnm)       /* Read a line from the file           */
    parse var inln key "," val
    reccnt = reccnt + 1
    say reccnt || ": " || key || ' => ' || val
    dbm~put(box('String', key)~getBytes(), box('String', val)~getBytes())
  end

  say
  say 'Closing QDBM File ...' 
  dbm~close()
  return

::requires bsf.cls /* BSF4Rexx Class File */