-- PROGRAM-NAME: ldtchash.rex        BACKUP-REQUIRED: Yes            
-- AUTHOR: Tony Dycks                REVISED BY: Tony Dycks          
-- DATE-WRITTEN: August 22, 2020     DATE-REVISED: August 22, 2020     
-- LANGUAGE: BSF4ooRexx              VERSION NO.: v641 for ooRexx v4.2+              
-- PLATFORM: Windows/Linux (Console) VERSION NO.: Java 8 SE or JDK 
--                                                                   
-- DESCRIPTION:                                                      
--    Load CSV Records Into A New TC Hash DB For The Command Line Arguments
--    Specifing The File Name of The Tokyo Cabinet Database File & CSV File                                          
--
-- PRE-REQUISITE(S):
--   1. Installation of Tokyo Cabinet Library API Dev Library & Dependencies
--      sudo apt-get install tokyocabinet-dev
--      sudo yum install tokyocabinet-devel 
--   or Build of Tokyo Cabinet API Library Source
--      su (for /usr/local configure and make)
--      ./configure
--      make
--      make check
--      make install
--      cp /usr/local/lib/libqdbm* /usr/lib
--   2. Setup of Tokyo Cabinet Library For Java Package
--      sudo apt-get install qdbm-java  {Debian Linux}
--   or Build of Tokyo Cabinet Library Source For Java 
--      cd java
--      su
--      Modify configure.in File for $JAVA_HOME
--      ./configure
--      make
--      make check
--      make install
--      cp /usr/local/lib/*qdbm* /usr/lib
--   3. Copy toykocabinet.jar File to $JAVA_HOME/jre/lib/ext Directory
--      cp /usr/local/lib/tokyocabinet.jar $JAVA_HOME/jre/lib/ext
--
-- REVISION HISTORY:                                                         
--   Version  Description                                                    
--   -------  -------------------------------------------------------------  
--     1.0    Original Implementation                                        
--            T. Dycks -- August 22, 2020
--     
-- USAGE:                                                            
--   sh ./rexxj2.sh ldtchash.rex <TCHashDB-Filename> <CSV-Filename> <Enter> {Linux}
--                                                                   
-- USAGE EXAMPLE:                                                    
--   sh ./rexxj2.sh ldtchash.rex $HOME/tokyocab/appl/MensWorldCupChamps.tch 
--     $HOME/txtfiles/MensWorldCupWinners.txt <Enter> {Linux}   
--                                                                   
-- REFERENCE(S):
--   Tokyo Cabinet: a modern implementation of DBM
--   Copyright (C) 2006-2011 FAL Labs
--   Last Update: Thu, 05 Aug 2010 
-- URL:
--   https://fallabs.com/tokyocabinet/
-- ================================================================================
--   Fallabs.com -- Presentation -- Introduction to Tokyo Products PDF 
-- URL:
--   https://fallabs.com/tokyocabinet/tokyoproducts.pdf
-- ================================================================================
--   Fallabs.com -- Specifications of the Java API
-- URL:
--   https://fallabs.com/tokyocabinet/javadoc/

if rxFuncQuery("BSF") then  /* no bsf4rexx support available yet, hence load Java */
do
   call rxFuncAdd "BsfLoadFuncs", "BSF4Rexx", "BsfLoadFuncs"
   call BsfLoadFuncs    /* load all BSF-functions                    */
   call BsfLoadJava     /* load the JVM and set up BSF4Rexx-support  */
end

if      BsfInvokedBy()=1 then say "This Rexx program was invoked by Java!"
else if BsfInvokedBy()=2 then say "This Rexx program was invoked by Rexx, JVM loaded by Rexx!"
else                          say "No JVM present, we got troubles ..."

parse arg TCHDBFile csvflnm

call banner
call main
call endmsg
exit

banner:
  say ' '
  say 'ldtchash.rex -- BSF4ooRexx Load TC Hash DB Key Values For CSV File'
  say 'Entered On Command Line'
  say 'Version 1.1'
  say 'Written By: Tony Dycks'
  say 'Revised By: Tony Dycks'
  say 'Date Written: August 22, 2020'
  say 'Date Revised: August 22, 2020'
  say ' '
  say 'TC Hash DB Filename: ' || TCHDBFile
  say 'CSV Filename: ' || csvflnm
  say ' '
  return

createtchash: procedure expose TCHDBFile
  say 'Creating TC Hash File ...'
  if getosname() = "Linux" then
    do
      touchcmd = 'touch ' || TCHDBFile
      address 'bash' touchcmd
    end
  else
    do
      touchcmd = 'fsutil file createnew ' || TCHDBFile ' 0'
      address 'cmd' touchcmd
    end
    
  say 'QDBM File Created ...'
  return
  
deltchashfile: procedure expose argqdbmflnm
  say 'Deleting TC Hash File ...'
  if getosname() = "Linux" then
    do
      delcmd = 'rm ' || argqdbmflnm
      address 'bash' delcmd
    end
  else
    do
      delcmd = 'del ' || argqdbmflnm
      address 'cmd' delcmd
    end
    
  say 'Deletion Completed ...'
  return
  
endmsg:
  say ' '
  say '>>> End Of Program --  ldtchash.rex <<<'
  say ' '
  return

fileexists: procedure expose TCHDBFile
  -- Check If Tokyo Cabinet Hash File Exists?
  -- If Exists; Just Open The Hash File in Writer Mode
  -- Otherwise Create A Blank Tokyo Cabinet Hash File
  File = bsf.import('java.io.File')
  tchfile = File~new(TCHDBFile)
    
  if tchfile~exists() then
    do
      say 'File Exists ...'
    end
  else
    do
      call createtchash
    end
    
  return

getosname: procedure
  system = bsf.loadClass("java.lang.System")
  return system~getProperty("os.name")  
  
-- Main Routine --
main: 
  reccnt = 0
  --osname = getosname()
  say 'Checking If TC Hash File Exists ...'
  call fileexists(TCHDBFile)
  say 'Opening Tokyo Cabinet Hash File ...'
  /*  TCH DB Access Module (Jar Files in JRE Lib Extensions Directory */
  HDB = bsf.import("tokyocabinet.HDB")  /* Import Java Class "tokyocabinet.HDB" */

  tchdb = HDB~new()
  tchdb~open(TCHDBFile, HDB~OWRITER)
  say 'Loading Hash DB Entries ...'
  say
  dbkey = ''
  dbval = ''
  reccnt = 0

  do while lines(csvflnm) <> 0
  	csvrec = linein(csvflnm)
  	parse var csvrec dbkey ',' dbval
    tchdb~put(dbkey, dbval)
    reccnt = reccnt + 1
    say reccnt || ': Key: ' || dbkey || ' => Value: ' || dbval 
  end
  
  say
  say 'Records Processed: ' || reccnt

  tchdb~close()
  return

::requires bsf.cls /* BSF4Rexx Class File */